local ADDON_NAME, _ = ...

---@class BrokerAnything
local BrokerAnything = LibStub("AceAddon-3.0"):NewAddon(ADDON_NAME, "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME)

BrokerAnything.Colors = {
	WHITE = "|cFFFFFFFF",
	RED = "|cFFDC2924",
	YELLOW = "|cFFFFF244",
	GREEN = "|cFF3DDC53",
	ORANGE = "|cFFE77324",
}

local Colors = BrokerAnything.Colors

BrokerAnything.MenuSeparator = {
	text = "", -- required!
	hasArrow = false,
	dist = 0,
	isTitle = true,
	isUninteractable = true,
	notCheckable = true,
	iconOnly = true,
	icon = "Interface\\Common\\UI-TooltipDivider-Transparent",
	tCoordLeft = 0,
	tCoordRight = 1,
	tCoordTop = 0,
	tCoordBottom = 1,
	tSizeX = 0,
	tSizeY = 8,
	tFitDropDownSizeX = true,
	iconInfo = {
		tCoordLeft = 0,
		tCoordRight = 1,
		tCoordTop = 0,
		tCoordBottom = 1,
		tSizeX = 0,
		tSizeY = 8,
		tFitDropDownSizeX = true
	},
}

function BrokerAnything:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("BrokerAnythingDB", {}, "Default")

	self.options = {
		type = "group",
		name = "BrokerAnything",
		args = {}
	}

	self.options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.options.args.profile.order = -1

	local mergeTable = LibStub("LibElioteUtils-1.0").mergeTable

	for _, module in self:IterateModules() do
		if (module.GetOptions) then
			mergeTable(self.options.args, module:GetOptions())
		end
	end

	local AceConfig = LibStub("AceConfig-3.0")
	AceConfig:RegisterOptionsTable(ADDON_NAME, self.options, { "brokerany", "ba" })

	local AceDialog = LibStub("AceConfigDialog-3.0")
	self.optionsFrame = AceDialog:AddToBlizOptions(ADDON_NAME)
end

function BrokerAnything:FormatBalance(value, tooltip)
	local text = ""
	if value > 0 then
		text = Colors.GREEN .. value .. "|r"
	end
	if value < 0 then
		text = Colors.RED .. value .. "|r"
	end

	if (tooltip) then
		if value == 0 then
			text = Colors.WHITE .. value .. "|r"
		end
		return text
	else
		if value == 0 then
			return ""
		end
		return " " .. Colors.WHITE .. "[" .. text .. Colors.WHITE .. "]"
	end
end

local registeredClicks = {}
local dropdownFrame = CreateFrame("Frame", "BrokerAnythingTitan_MenuFrame", nil, "UIDropDownMenuTemplate")
function BrokerAnything:RegisterOnClick(onClick)
	table.insert(registeredClicks, onClick)
end

function BrokerAnything:UnregisterOnClick(onClick)
	for i, v in ipairs(registeredClicks) do
		if (onClick == v) then
			table.remove(registeredClicks, i)
		end
	end
end

function BrokerAnything:CreateOnClick(onClick)
	return function(...)
		local menu
		local type = type(onClick)
		if (type == "function") then
			menu = onClick(...)
		else
			menu = {}
		end

		for _, v in ipairs(registeredClicks) do
			v(menu, ...)
		end

		table.insert(menu, BrokerAnything.MenuSeparator)
		table.insert(menu, {
			notCheckable = true,
			text = CANCEL,
			keepShownOnClick = false
		})

		L_EasyMenu(menu, dropdownFrame, "cursor", 0, 0, "MENU")
	end
end

BrokerAnything.DefaultOnClick = BrokerAnything:CreateOnClick()

function BrokerAnything:FormatBoolean(b)
	if b then return L["Yes"] else return L["No"] end
end
